#include <windows.h>
#include "dxl_hal.h"
#include "roboard_dll.h"


#define LATENCY_TIME	1		// ms


float gfByteTransTime = 0.0f;
float gfRcvWaitTime = 0.0f;
LARGE_INTEGER gStartTime;


int dxl_hal_open()
{
	dxl_hal_close();
	// Initialize com_port
	if( com2_Init(COM_HDUPLEX) )
	{
		if( com2_SetFormat(COM_BYTESIZE8,COM_STOPBIT1,COM_NOPARITY) )
			return 1;
	}

	dxl_hal_close();
	return 0;
}

void dxl_hal_close()
{
	if( com2_InUse() )
		com2_Close();
}

/* In this library, "dxl_hal_set_baud" only suppot several baud rates:
 * 748800BPS
 * 499200BPS
 * 249600BPS
 * 115200BPS
 * 57600BPS
 * 38400BPS
 * 19200BPS
 * 9600BPS
 * Please set parameter "baudrate" to be close to those baud rates.
 */
int dxl_hal_set_baud( float baudrate )
{
	unsigned int iBaudrate;
	if( baudrate > 666666 )
	{
		iBaudrate = COMBAUD_748800BPS;
		baudrate = 748800;
	}
	else if( baudrate<510000 && baudrate>490000 )
	{
		iBaudrate = COMBAUD_499200BPS;
		baudrate = 499200;
	}
	else if( baudrate<260000 && baudrate>240000 )
	{
		iBaudrate = COMBAUD_249600BPS;
		baudrate = 249600;
	}
	else if( baudrate<120000 && baudrate>110000 )
	{
		iBaudrate = COMBAUD_115200BPS;
		baudrate = 115200;
	}
	else if( baudrate<62600 && baudrate>52600 )
	{
		iBaudrate = COMBAUD_57600BPS;
		baudrate = 57600;
	}
	else if( baudrate<43400 && baudrate>33400 )
	{
		iBaudrate = COMBAUD_38400BPS;
		baudrate = 38400;
	}
	else if( baudrate<21200 && baudrate>17200 )
	{
		iBaudrate = COMBAUD_19200BPS;
		baudrate = 19200;
	}
	else if( baudrate<9700 && baudrate>9500 )
	{
		iBaudrate = COMBAUD_9600BPS;
		baudrate = 9600;
	}
	if( com2_SetBaud(iBaudrate) )
	{
		gfByteTransTime = ((1000.0f / baudrate) * 12.0f);
		return 1;
	}
	return 0;
}

void dxl_hal_clear(void)
{
	com2_ClearRFIFO();
}

int dxl_hal_tx( unsigned char *pPacket, int numPacket )
{
	if( com2_Send(pPacket,numPacket) )
		return numPacket;
	return 0;
}

int dxl_hal_rx( unsigned char *pPacket, int numPacket )
{
	if( com2_Receive(pPacket, numPacket) )
		return numPacket;
	return 0;
}

void dxl_hal_set_timeout( int NumRcvByte )
{
	QueryPerformanceCounter( &gStartTime );
	gfRcvWaitTime = (float)(gfByteTransTime*(float)NumRcvByte + 2*LATENCY_TIME + 2.0f);
}

int dxl_hal_timeout(void)
{
	LARGE_INTEGER end, freq;
	double time;

	QueryPerformanceCounter( &end );
	QueryPerformanceFrequency( &freq );

	time = (double)(end.QuadPart - gStartTime.QuadPart) / (double)freq.QuadPart;
	time *= 1000.0;

	if( time > gfRcvWaitTime )
		return 1;
	else if( time < 0 )
		QueryPerformanceCounter( &gStartTime );

	return 0;
}
